unit sha1obj;

interface

uses
  KOL;

type
(*
 * \brief          SHA-1 context structure
 *)
SHA1_CONTEXT = packed record
  total: Array[0..1] of LongWord;
  state: Array[0..4] of LongWord;
  buffer: Array[0..63] of Byte;
  ipad: Array[0..63] of Byte;
  opad: Array[0..64] of Byte;
end;
PSHA1_CONTEXT = ^SHA1_CONTEXT;
SHA1_DIGEST = Array [0..19] of Byte;
PSHA1_DIGEST = ^SHA1_DIGEST;

function _memcmp (s1,s2: Pointer; numBytes: LongWord): integer; cdecl;
procedure _memcpy (s1, s2: Pointer; n: Integer); cdecl;
procedure _memmove(dstP, srcP: pointer; numBytes: LongWord); cdecl;
procedure _memset (s: Pointer; c: Byte; n: Integer); cdecl;

function _puts(content{string}: PChar):Integer; cdecl;
          external 'msvcrt.dll' name 'puts';
function _fopen(filename: PChar; mode: PChar):Pointer; cdecl;
          external 'msvcrt.dll' name 'fopen';
function _fclose(stream: Pointer):Integer; cdecl;
          external 'msvcrt.dll' name 'fclose';
function _fread(buffer: Pointer; size, count: Cardinal; stream: Pointer):LongBool; cdecl;
          external 'msvcrt.dll' name 'fread';

{$LINK 'sha1_gcc_omf.o'}

(**
 * \brief          SHA-1 context setup
 *
 * \param ctx      context to be initialized
 *)
procedure _sha1_starts( ctx: PSHA1_CONTEXT ); cdecl; external;

(**
 * \brief          SHA-1 process buffer
 *
 * \param ctx      SHA-1 context
 * \param input    buffer holding the  data
 * \param ilen     length of the input data
 *)
procedure _sha1_update( ctx: PSHA1_CONTEXT; input: PChar; ilen: Cardinal ); cdecl; external;

(**
 * \brief          SHA-1 final digest
 *
 * \param ctx      SHA-1 context
 * \param output   SHA-1 checksum result
 *)
procedure _sha1_finish( ctx: PSHA1_CONTEXT; var output: SHA1_DIGEST ); cdecl; external;

(*
 * \brief          Output = SHA-1( input buffer )
 *
 * \param input    buffer holding the  data
 * \param ilen     length of the input data
 * \param output   SHA-1 checksum result
 *)
procedure _sha1( input: Pointer; ilen: Integer; var output: SHA1_DIGEST ); cdecl; external;

(*KOL Related Addon Function*)
function SHA1Stream(Input: PStream):SHA1_DIGEST;

var
sha1_starts: procedure( ctx: PSHA1_CONTEXT ) cdecl = _sha1_starts;
sha1_update: procedure( ctx: PSHA1_CONTEXT; input: PChar; ilen: Cardinal ) cdecl = _sha1_update;
sha1_finish: procedure( ctx: PSHA1_CONTEXT; var output: SHA1_DIGEST ) cdecl = _sha1_finish;
sha1: procedure( input: Pointer; ilen: Integer; var output: SHA1_DIGEST ) cdecl = _sha1;


implementation

function SHA1Stream(Input: PStream):SHA1_DIGEST;
var
  n: Integer;
  ctx: SHA1_CONTEXT;
  buf: Array of Byte;
begin
  SetLength(buf, 1024);

  _sha1_starts(@ctx);

  while (Input.Position < Input.Size) do
  begin
    n := Input.Read(buf[0], Length(buf));
    _sha1_update(@ctx, @buf[0], n);
  end;

  _sha1_finish(@ctx, Result);

  FillChar(ctx, sizeof(ctx), 0);

  SetLength(buf, 0);
end;

procedure _memset(s: Pointer; c: Byte; n: Integer); cdecl;
begin
  FillChar(s^, n, c);
end;

procedure _memcpy(s1, s2: Pointer; n: Integer); cdecl;
begin
  Move(s2^, s1^, n);
end;

function _memcmp (s1, s2: Pointer; numBytes: LongWord): integer; cdecl;
var
  i: integer;
  p1, p2: ^byte;
begin
  p1 := s1;
  p2 := s2;
  for i := 0 to numBytes -1 do
  begin
    if p1^ <> p2^ then
    begin
      if p1^ < p2^ then
        Result := -1
      else
        Result := 1;
      exit;
    end;
    inc(p1);
    inc(p2);
  end;
  Result := 0;
end;

procedure _memmove(dstP, srcP: pointer; numBytes: LongWord); cdecl; 
begin 
  Move(srcP^, dstP^, numBytes); 
  FreeMem(srcP, numBytes); 
end;

end.


